using FooTheory.CodeCamp.DI.Interfaces.DataAccess;
using FooTheory.CodeCamp.DI.Interfaces.Domain;
using FooTheory.CodeCamp.DI.Interfaces.Services;
using FooTheory.CodeCamp.DI.Interfaces.Services.Mappers;
using FooTheory.CodeCamp.DI.Services;
using NUnit.Framework;
using Rhino.Mocks;
using Rhino.Mocks.Constraints;

namespace FooTheory.CodeCamp.DI.UnitTests.Services
{
    [TestFixture]
    public class CustomerServiceTests
    {
        private MockRepository mockery;

        private ICustomerRepository repository;
        private ICustomerDTOMapper mapper;
        private CustomerService service;


        [SetUp]
        public void Setup()
        {
            mockery = new MockRepository();
            repository = mockery.DynamicMock<ICustomerRepository>();
            mapper = mockery.DynamicMock<ICustomerDTOMapper>();

        }

        [Test]
        public void GetACustomerFrom_an_id_should_call_the_repository_and_mapper()
        {
            // Interaction based test to ensure that the service layer is calling 
            // the other components as I expect them to. 

            With.Mocks(mockery)
            .Expecting(delegate
            {
                // We're exepecting that the repository will get called. 
                // I dont care what the ID is (which is why I say IgnoreArguments)
                // but I do want to know that its greater than zero
                Expect.Call(repository.GetFrom(1))
                .IgnoreArguments()
                .Constraints(Is.GreaterThan(0))
                .Return(null);

                // We're expecting that the mapper gets called, and when it does
                // we dont care about the arguments because we're just verifying
                // interactions. 
                Expect.Call(mapper.MapFrom(null))
                .IgnoreArguments()
                .Return(null);

            })
            .Verify(delegate
            {
                // Lets fire it off
                service = new CustomerService(repository, mapper);
                service.GetACustomerFrom(1);
            });
        }
    }
}